//
// 
// File: dso.c
// 
// Copyright (C) 2007, Jerry O'Keefe

/* 
  indent -gnu -br -cli2 -lp -nut dso.c
*/

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/time.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <stdlib.h>
#include <sys/shm.h>
#include <sys/ipc.h>
#include <signal.h>
#include <gtk/gtk.h>

#include "dso_io.h"

int bufsiz = 1000;
#define BUF_SIZE (4096)
uint8 inbuf[BUF_SIZE];
uint8 inbuf2[BUF_SIZE];

GtkWidget *spinner1;
GtkWidget *spinner2;
GtkWidget *spinner3;
GtkWidget *slope_button;
GtkWidget *trigged_button;
GtkWidget *dot_button;
GtkWidget *noise_button;
GtkWidget *statusbar;

GdkColor blue;
GdkColor red;
GdkColor green;
GdkColor yellow;
GdkColor grey;
GdkColor white;
GdkColor black;

GtkItemFactory *item_factory;
int lastSample[1000];
int lastSample1[1000];

shmLayout *pShm = NULL;
pid_t pID = 0;
int shmid = 0;
int set_dso_x = 0;
int set_dso_y = 0;
int set_dso_x1 = 510;
int set_dso_y1 = 255;
int SampleCount = 0;
int dso_debug = 0;
int dso_noise = 0;
char Version_Str[]={"0.01"};

gint draw_scope_scale (GtkWidget * widget);

static void
dummy_callback(gpointer data, guint callback_action, GtkWidget * w)
{    
    if (dso_debug) {
	   g_message ("Hello, World!\n");
    }
}

static gint
enable_delete (GtkWidget * widget, GdkEvent * event, gpointer data)
{
  gtk_main_quit ();
  return 0;
}

static gboolean
button_press_event (GtkWidget * widget, GdkEventButton * event, gpointer data)
{                              
  /* wait for mouse button press event */
  if (event->button == 1) {
    // save last marker location
    set_dso_x1 = set_dso_x;   
    set_dso_y1 = set_dso_y;

    // get new save marker location
    set_dso_x = (int) event->x;
    set_dso_y = (int) event->y;
    if (dso_debug) {
       g_message (" button: dso_x: %d, set_dso_y: %d\n", set_dso_x, set_dso_y);
    }
  }
  return TRUE;
}

void
DestroyNotifyFunc (gpointer data)
{
  printf ("data is 0x%08lX\n", (long) data);
}

static GtkItemFactoryEntry menu_items[] = {
  {"/_File", NULL, NULL, 0, "<Branch>"},
  {"/File/_New", "<control>N", dummy_callback, 2, NULL},
  {"/File/_Open", "<control>O", dummy_callback, 0, NULL},
  {"/File/_Save", "<control>S", dummy_callback, 0, NULL},
  {"/File/Save _As",   NULL, NULL, 0, NULL},
  {"/File/sep1",       NULL, NULL, 0, "<Separator>"},
  {"/File/Quit", "<control>Q", gtk_main_quit, 0, NULL},
  {"/_Options",        NULL, NULL, 0, "<Branch>"},
  {"/Options/hello",   NULL, dummy_callback, 1, "<CheckItem>"},
  {"/Options/Option2", NULL, dummy_callback, 2, "<ToggleItem>"},
  {"/Options/Option3", NULL, dummy_callback, 3, "<CheckItem>"},
  {"/Options/Option4", NULL, dummy_callback, 4, "<ToggleItem>"},
  {"/_Help",           NULL, NULL, 0, "<LastBranch>"},
  {"/Help/One",        NULL, NULL, 0, NULL},
  {"/Help/Two",        NULL, NULL, 0, "<Branch>"},
  {"/Help/Two/A",      NULL, NULL, 0, "<RadioItem>"},
  {"/Help/Two/B",      NULL, NULL, 0, "/Help/Two/A"},
  {"/Help/Two/C",      NULL, NULL, 0, "/Help/Two/A"},
  {"/Help/Two/D",      NULL, NULL, 0, "/Help/Two/A"},
  {"/Help/Two/E",      NULL, NULL, 0, "/Help/Two/A"},
  {"/Help/Three",      NULL, NULL, 0, NULL},
};

void
get_main_menu (GtkWidget * window, GtkWidget ** menubar)
{
  GtkAccelGroup *accel_group;
  
  gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);
  accel_group = gtk_accel_group_new ();
  item_factory =
    gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>", accel_group);

  gtk_item_factory_create_items (item_factory, nmenu_items, menu_items,
                                 "foo");
  if (menubar) {
    *menubar = gtk_item_factory_get_widget (item_factory, "<main>");
  }
}

/*******************************************************/
/*						                               */
/*******************************************************/

int
init_colors (GtkWidget * widget)
{
  GdkColormap *colormap;

  colormap = gtk_widget_get_colormap (widget);
  if (!colormap) {
    return -1;
  }

  gdk_color_parse ("blue", &blue);
  gdk_colormap_alloc_color (colormap, &blue, FALSE, TRUE);

  gdk_color_parse ("green", &green);
  gdk_colormap_alloc_color (colormap, &green, FALSE, TRUE);

  gdk_color_parse ("red", &red);
  gdk_colormap_alloc_color (colormap, &red, FALSE, TRUE);

  gdk_color_parse ("grey", &grey);
  gdk_colormap_alloc_color (colormap, &grey, FALSE, TRUE);

  gdk_color_parse ("yellow", &yellow);
  gdk_colormap_alloc_color (colormap, &yellow, FALSE, TRUE);

  gdk_color_parse ("white", &white);
  gdk_colormap_alloc_color (colormap, &white, FALSE, TRUE);

  gdk_color_parse ("black", &black);
  gdk_colormap_alloc_color (colormap, &black, FALSE, TRUE);
  return 0;
}

gint
timeout_callback (gpointer data)
{
  extern unsigned char serBuf[];
  memcpy (pShm->cmds, serBuf, 10);

  /* DC offset */
  pShm->cmds[4] =
    gtk_spin_button_get_value_as_int ((GtkSpinButton *) spinner2);

  /* Gain position */
  pShm->cmds[5] =
    gtk_spin_button_get_value_as_int ((GtkSpinButton *) spinner1);

  /* trigger point */
  pShm->cmds[0] =
    gtk_spin_button_get_value_as_int ((GtkSpinButton *) spinner3);

  pShm->cmds[2] = 0;
  if (gtk_toggle_button_get_active ((GtkToggleButton *) slope_button)) {
    pShm->cmds[2] |= 1;
  }
  if (!gtk_toggle_button_get_active ((GtkToggleButton *) trigged_button)) {
    pShm->cmds[2] |= 2;
  }

  pShm->captureStartFlag = 1;

  return TRUE;
}

gint
draw_marker (GtkWidget * widget, int set_dso_x, int set_dso_y)
{ 
  GdkGC *gc = NULL;
  
  gc = gdk_gc_new (widget->window);
  
  /* Draw makers */
  gdk_gc_set_foreground (gc, &green);
  gdk_draw_line (widget->window, gc, set_dso_x - 5, set_dso_y, set_dso_x + 5, set_dso_y);  // hoz line 
  gdk_draw_line (widget->window, gc, set_dso_x, set_dso_y - 5, set_dso_x, set_dso_y + 5);  // ver line 

  gdk_gc_set_foreground (gc, &blue);
  gdk_draw_point (widget->window, gc, set_dso_x, set_dso_y);
  g_object_unref (G_OBJECT (gc));

  return 0;
}

gint
draw_scope_scale (GtkWidget * widget)
{
  int width = widget->allocation.width;
  int height = widget->allocation.height;
  int offset = 0;
  GdkGC *gc = NULL;

  gc = gdk_gc_new (widget->window);
  gdk_gc_set_background (gc, &white);
  gdk_gc_set_foreground (gc, &black);

  /* White background */
  gdk_draw_rectangle (widget->window, widget->style->white_gc, TRUE, 0, 0,
                      width, height);
  /* Black border */
  gdk_draw_rectangle (widget->window, widget->style->black_gc, FALSE, 0, 0, 
                      width - 1, height - 1);

  gint8 dash_list[2];
  dash_list[0] = 1;
  dash_list[1] = 9;

  gdk_gc_set_dashes (gc, 0, dash_list, sizeof (dash_list));
  gdk_gc_set_line_attributes (gc, 1, GDK_LINE_DOUBLE_DASH, GDK_CAP_NOT_LAST,
                              GDK_JOIN_BEVEL);

  /* Draw hoz center line */
  gdk_draw_line (widget->window, widget->style->black_gc, 0, height / 2,
                 width, height / 2);

  /* Draw ver center line */
  gdk_draw_line (widget->window, widget->style->black_gc, width / 2, 0,
                 width / 2, height);

  /* Draw Hoz scales */
  for (offset = -100; offset <= 100; offset += 50) {
    if (offset != 0) {
      gdk_draw_line (widget->window, gc, 0, height / 2 + offset, width,
                     height / 2 + offset);
    }
  }

  /* Draw Ver scales */
  for (offset = -250; offset <= 250; offset += 50) {
    if (offset != 0) {
      gdk_draw_line (widget->window, gc, width / 2 + offset, 0,
                     width / 2 + offset, height);
    }
  }

  /* Draw zero and full scale data point lines */
  dash_list[0] = 10;
  dash_list[1] = 10;
  gdk_gc_set_dashes (gc, 0, dash_list, sizeof (dash_list));
  gdk_gc_set_foreground (gc, &green);
  offset = 127;
  gdk_draw_line (widget->window, gc, 0, height / 2 + offset, width,
                 height / 2 + offset);
  gdk_draw_line (widget->window, gc, 0, height / 2 - offset, width,
                 height / 2 - offset);

  gdk_gc_set_line_attributes (gc, 1, GDK_LINE_SOLID, GDK_CAP_NOT_LAST,
                              GDK_JOIN_BEVEL);

  gdk_gc_set_foreground (gc, &green);
  gdk_draw_line (widget->window, gc, 120, height / 2, 136, height / 2);
  gdk_draw_line (widget->window, gc, 127, height / 2 - 10, 127,
                 height / 2 + 10);

  g_object_unref (G_OBJECT (gc));

  return TRUE;
}

gint
draw_wave (GtkWidget * widget, uint8 * inbuf, int bufLen)
{
  int width = widget->allocation.width;
  int height = widget->allocation.height;
  int ii;
  int yvalue;
  int last_yvalue = 0;
  gint context_id;
  char statusStr[100];
  uint8 *pBuf;
  int dso_value;
  int dso_time;
  int rc=0;
  GdkGC *gc = NULL;

  gc = gdk_gc_new (widget->window);
  rc = draw_scope_scale(widget); 

  draw_marker (widget, set_dso_x, set_dso_y);
  draw_marker (widget, set_dso_x1, set_dso_y1);

  gdk_gc_set_foreground (gc, &red);

  pBuf = inbuf+1;
  bufsiz = width-1;

  /* Draw 1st signal */
  for (ii = 0; ii != bufsiz; ii++) {
    yvalue = (pBuf[ii] - 128);
    if (gtk_toggle_button_get_active ((GtkToggleButton *) noise_button)) {
        yvalue += (rand () & 0x7);
    }
    /* Move zero to center of window */
    yvalue = height / 2 - yvalue;
    if (ii) {
      if (gtk_toggle_button_get_active ((GtkToggleButton *) dot_button)) {
        gdk_draw_line (widget->window, gc, ii - 1, last_yvalue, ii, yvalue);
      }
      else {
        gdk_draw_point (widget->window, gc, ii, yvalue);
      }
    }
    last_yvalue = yvalue;
  }

  dso_time = abs (set_dso_x1 - set_dso_x);
  dso_value = abs (set_dso_y1 - set_dso_y);

  context_id =
    gtk_statusbar_get_context_id (GTK_STATUSBAR (statusbar), "Statusbar");
  sprintf (statusStr,
             "set_dso_x: %d, set_dso_y: %d,  value: %d, time: %d, %d nsec, %3.3f volts, freq: %6.1fHz",
             set_dso_x, set_dso_y, dso_value, dso_time, 10 * dso_time,
             ((float) dso_value) * 0.004545,
             100000000.0 / ((float) dso_time));

  gtk_statusbar_push (GTK_STATUSBAR (statusbar), context_id, statusStr);

  g_object_unref (G_OBJECT (gc));
  return TRUE;
}


gint
idle_callback (gpointer data)
{ 
  static int init=0;
  
  if (!init) {
      init = 1;  
      draw_scope_scale((GtkWidget *)data); 
      return 1;
  }
  if (pShm->captureDoneFlag != 0) {
    draw_wave ((GtkWidget *) data, pShm->data, pShm->dataLen);
    pShm->captureDoneFlag = 0;
  }
  return 1;
}

int
main (int argc, char **argv)
{
  GtkWidget *mainwindow, *drawingarea;
  GtkWidget *menubar,*vbox;
  int rc = 0;
  gint c;
  char serial_port[30];

  gtk_set_locale ();
  gtk_init (&argc, &argv);

  /* setup default serail port parameters */
  strcpy (serial_port, "/dev/ttyS0");

  while ((c = getopt (argc, argv, "vnd?hp:")) != EOF) {
    switch (c) {
      case 'n':
        dso_noise = 1;
        break;
      case 'd':
        dso_debug = 1;
        break;
      case 'p':
        if (optarg != NULL) {
          strcpy (serial_port, optarg);
        }
        break;
      case 'v':
        fprintf (stderr, "dso, Version: %s, Built: %s %s\n", Version_Str,__DATE__,__TIME__);
        exit (0);
        break;
      case '?':
      case 'h':
        fprintf (stderr, "example: ./dso -p %s\n", "/dev/ttyS0");
        fprintf (stderr, "example: ./dso -p %s\n", "/dev/ttyS1");
        fprintf (stderr, "example: ./dso -p %s\n", "/dev/ttyUSB0");
        fprintf (stderr, "default: ./dso -p %s\n", serial_port);
        exit (0);
        break;
    }
  }

  /* setup serial port */
  rc = dso_serial_init (serial_port);
  if (rc != 0) {
    fprintf (stderr, "Error Serial port: %s\n", serial_port);
    perror ("Serial port");
    exit (rc);
  }

  /* setup shared memory */
  shmid = shmget (IPC_PRIVATE, SHM_SIZE, IPC_CREAT);
  pShm = shmat (shmid, NULL, 0);

  /* start up serial IO process */
  pID = fork ();
  if (pID == 0) {               // child
    // Code only executed by child process
    rc = dsoGetSamples (serial_port);
    exit (rc);
  }
  else if (pID < 0) {           // failed to fork
    fprintf (stderr, "Failed to fork\n");
    perror ("serial IO");
    exit (1);
  }

  /* Setp the GTK+ GUI */
  mainwindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  init_colors (mainwindow);
  gtk_signal_connect (GTK_OBJECT (mainwindow), "destroy",
                      GTK_SIGNAL_FUNC (gtk_main_quit), NULL);
  gtk_signal_connect (GTK_OBJECT (mainwindow), "delete_event",
                      GTK_SIGNAL_FUNC (enable_delete), NULL);
  gtk_window_set_title (GTK_WINDOW (mainwindow), "FPGA_DSO");
  gtk_container_border_width (GTK_CONTAINER (mainwindow), 5);

  get_main_menu (mainwindow, &menubar);
  vbox = gtk_vbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (vbox), menubar, FALSE, TRUE, 0);
  gtk_widget_show (menubar);

  gtk_container_add (GTK_CONTAINER (mainwindow), vbox);
  gtk_widget_show (vbox);

  /******************************/
  GtkWidget *hbox1;
  hbox1 = gtk_hbox_new (FALSE, 0);

  gtk_container_border_width (GTK_CONTAINER (hbox1), 1);
  gtk_container_add (GTK_CONTAINER (vbox), hbox1);
  gtk_widget_show (hbox1);

  /******************************/
  drawingarea = gtk_drawing_area_new ();
  gtk_drawing_area_size ((GtkDrawingArea *) drawingarea, 512, 300);
  gtk_container_add (GTK_CONTAINER (hbox1), drawingarea);
  gtk_widget_show (mainwindow);

  GtkWidget *vbox3;
  vbox3 = gtk_vbox_new (FALSE, 0);

  gtk_container_border_width (GTK_CONTAINER (vbox3), 5);
  gtk_container_add (GTK_CONTAINER (hbox1), vbox3);
  gtk_widget_show (vbox3);

  /*****************/
  /* create slope check box */
  slope_button = gtk_check_button_new_with_label ("+Slope");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (slope_button), TRUE);
  gtk_box_pack_start (GTK_BOX (vbox3), slope_button, TRUE, TRUE, 0);

  /* create trigger check box */
  trigged_button = gtk_check_button_new_with_label ("Triggered");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (trigged_button), TRUE);
  gtk_box_pack_start (GTK_BOX (vbox3), trigged_button, TRUE, TRUE, 0);

  /* create dot-line check box */
  dot_button = gtk_check_button_new_with_label ("Line/Dot");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dot_button), TRUE);
  gtk_box_pack_start (GTK_BOX (vbox3), dot_button, TRUE, TRUE, 0);

  /* create noise button  */
  noise_button = gtk_check_button_new_with_label ("Noise");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (noise_button), FALSE);
  gtk_box_pack_start (GTK_BOX (vbox3), noise_button, TRUE, TRUE, 0);

  GtkWidget *separator;
  separator = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (vbox3), separator, FALSE, TRUE, 0);
  gtk_widget_show (separator);

  /************/
  GtkWidget *label3;
  label3 = gtk_label_new (("DC Offset"));
  gtk_widget_show (label3);
  gtk_box_pack_start (GTK_BOX (vbox3), label3, FALSE, FALSE, 0);

  GtkAdjustment *adj1;
  adj1 = (GtkAdjustment *) gtk_adjustment_new (110.0, 0.0, 255.0, 1.0, 1.0, 1.0);
  spinner1 = gtk_spin_button_new (GTK_ADJUSTMENT (adj1), 0.0, 0.0);
  gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (spinner1), TRUE);
  gtk_box_pack_start (GTK_BOX (vbox3), spinner1, TRUE, TRUE, 0);

  /************/
  GtkWidget *label4;
  label4 = gtk_label_new (("Gain"));
  gtk_widget_show (label4);
  gtk_box_pack_start (GTK_BOX (vbox3), label4, FALSE, FALSE, 0);

  GtkAdjustment *adj2;
  adj2 = (GtkAdjustment *) gtk_adjustment_new (116.0, 0.0, 255.0, 1.0, 1.0, 1.0);
  spinner2 = gtk_spin_button_new (GTK_ADJUSTMENT (adj2), 0.0, 0.0);
  gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (spinner2), TRUE);
  gtk_box_pack_start (GTK_BOX (vbox3), spinner2, TRUE, TRUE, 0);

  /************/
  GtkWidget *label5;
  label5 = gtk_label_new (("Trigger Point"));
  gtk_widget_show (label5);
  gtk_box_pack_start (GTK_BOX (vbox3), label5, FALSE, FALSE, 0);

  GtkAdjustment *adj3;
  adj3 = (GtkAdjustment *) gtk_adjustment_new (116.0, 0.0, 255.0, 1.0, 1.0, 1.0);
  spinner3 = gtk_spin_button_new (GTK_ADJUSTMENT (adj3), 0.0, 0.0);
  gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (spinner3), TRUE);
  gtk_box_pack_start (GTK_BOX (vbox3), spinner3, TRUE, TRUE, 0);

  gtk_widget_show_all (mainwindow);

  gtk_timeout_add (75, timeout_callback, (gpointer) drawingarea);

  statusbar = gtk_statusbar_new ();
  gtk_object_set_data (GTK_OBJECT (mainwindow), "statusbar", statusbar);
  gtk_box_pack_start (GTK_BOX (vbox), statusbar, FALSE, FALSE, 0);

  gtk_widget_show (statusbar);

  g_signal_connect (G_OBJECT (drawingarea), "motion_notify_event",
                    GTK_SIGNAL_FUNC (button_press_event), NULL);
  g_signal_connect (G_OBJECT (drawingarea), "button_press_event",
                    GTK_SIGNAL_FUNC (button_press_event), NULL);

  gtk_widget_add_events (drawingarea,
                         GDK_BUTTON_PRESS_MASK | GDK_POINTER_MOTION_MASK);

  /********* Idle function **********/
  gtk_idle_add (idle_callback, (gpointer) drawingarea);

  gtk_main ();

  /* gtk_main has exit, time to clear up */

  dso_serial_close ();

  pShm->quitFlag = 1;
  kill (pID, 9);     // kill dsoGetSamples()

  return 0;
}
