//
// 
// File: dso_io.c
// 
// Copyright (C) 2007, Jerry O'Keefe


#include <termios.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <sys/signal.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/shm.h>
#include <sys/types.h>
#include <sys/ipc.h>

#include "dso_io.h"

extern int shmid;
extern int dso_debug;
int comfd;
struct termios oldtio, newtio;  //place for old and new port settings for serial port

int
dso_serial_init (char *devicename)
{
  int speed = B115200;
  int flags;

  if (dso_debug) {
     fprintf (stderr, "Serial port: %s\n", devicename);
  }
  comfd = open (devicename, O_RDWR | O_NOCTTY | O_NONBLOCK);
  if (comfd < 0) {
    fprintf (stderr, "Error from dso_serial_init(), %d\n", comfd);
    perror (devicename);
    return -1;
  }
  tcgetattr (comfd, &oldtio);   // save current port settings 
  newtio.c_cflag = speed | CS8 | CREAD;
  newtio.c_iflag = IGNPAR;
  newtio.c_oflag = 0;
  newtio.c_lflag = 0;
  newtio.c_cc[VMIN] = 1;
  newtio.c_cc[VTIME] = 0;
  tcflush (comfd, TCIFLUSH);
  tcsetattr (comfd, TCSANOW, &newtio);

  /* The set RTS/DTR to power the TXDI's tx line  */
  //get line bits for serial port
  ioctl (comfd, TIOCMGET, &flags);

  //make sure RTS and DTR lines are high
  flags &= ~TIOCM_DTR;
  flags |= TIOCM_RTS;
  ioctl (comfd, TIOCMSET, &flags);

  return 0;
}

int
dso_serial_close ()
{
  tcsetattr (comfd, TCSANOW, &oldtio);
  close (comfd);
  return 0;
}

//unsigned char serBuf[] = {128,0,128,0,224,226,221,186,128,0};
unsigned char serBuf[] = { 128, 0, 128, 0, 224, 226, 221, 186, 128, 0 };

#if 0
index 0: free run / auto and trigger level index 1: sample rate index 2:
index 3:
index 4: 1 st channel DC offset ? ? index 5 : index 6 : index 7:2 nd channel DC offset ? ?
  index 8 :
  index 9 :
#endif

static int
dso_serial_send (uint8 * pCnds, int cmdLen)
{
  tcflush (comfd, TCIFLUSH);
  write (comfd, pCnds, cmdLen);
  return 0;
}

static int
dso_serial_get (unsigned char *pBug, int bufLen)
{
  int i;
  int rc = 0;
  int count = 0;
  unsigned char c;

  memset (pBug, 0, bufLen);
  for (i = 0; i < 1000; i++);

  for (i = 0; i < bufLen; i++) {
    count = 0;;
    do {
      rc = read (comfd, &c, 1);
      count++;
      if (count > 10000) {
        if (i == 0) {
          return -1;
        }
        return i;
      }
    } while (rc == -1);
    pBug[i] = c;
  }
  return i;
}

int
dsoGetSamples ()
{
  shmLayout *pShm;
  int sample_cnt;
  int update_cnt = 0;

  pShm = (shmLayout *) shmat (shmid, NULL, 0);
  if (pShm == NULL) {
     fprintf (stderr, "Shared memory pointer=NULL\n");
     return -1;
  }
  memset (pShm, 0, 1000);
  pShm->quitFlag = 0;
  pShm->captureStartFlag = 0;
  pShm->captureDoneFlag = 0;
  if (dso_debug) {
     printf ("shmid: %d, ptr: 0x%08lX\n", shmid, (unsigned long) pShm);
  }
  while (pShm->quitFlag == 0) {
    if (pShm->captureStartFlag != 0) {
      pShm->captureBusyFlag = 1;
      update_cnt++;
      dso_serial_send (pShm->cmds, 10);
      sample_cnt = dso_serial_get (pShm->data, 600);
      pShm->dataLen = sample_cnt;
//        printf("sample_cnt: %d, update_cnt: %d, 0x%08lX\n",sample_cnt,update_cnt,(unsigned long)pShm);
//        printf("update_cnt: %d, sample_cnt: %d\n",update_cnt,sample_cnt);
//        byteDump(ptr+2,128,0);       
      pShm->captureBusyFlag = 0;
      pShm->captureStartFlag = 0;
      pShm->captureDoneFlag = 1;
    }
  }

  shmdt (pShm);
  return 0;
}
